#include "common.h"
extern volatile uint32_t ttms;
static volatile uint32_t cntr;

void _delay_ms(uint32_t dtime)
{
  uint32_t dtms = 0;
  dtms = ttms;
  while (dtms > ttms || ttms - dtms > dtime) { asm("nop"); };
}

void delay_us(uint16_t i) 
{
  for (; i > 0; i--) {
    for (uint16_t j = 0; j < 1000; ++j) {
      __asm__ __volatile__("nop\n\t" ::
                               : "memory");
    }
  }
}

void gpio_init(void)
{
  RCC->AHBENR  |= RCC_AHBENR_GPIOAEN  | RCC_AHBENR_GPIOBEN; // Enable clocks to the GPIO subsystems (A&B)
  GPIOB->MODER |= GPIO_MODER_MODER3_0;
  GPIOA->MODER |= GPIO_MODER_MODER15_0;
  // Настройка GPIO
  // Сброс выходов
  // GPIOA->ODR = 0;
  // GPIOB->ODR = 0;
}

void systick_init(void)
{
  SysTick->LOAD = TimerTick;
  SysTick->VAL  = TimerTick;
  SysTick->CTRL = SysTick_CTRL_CLKSOURCE_Msk | SysTick_CTRL_TICKINT_Msk | SysTick_CTRL_ENABLE_Msk;;
}

void rcc_sysclockinit(void)
{
  RCC->CR = RCC_CR_HSEON;
  while(! (RCC->CR & RCC_CR_HSERDY));
  RCC->CFGR2 = RCC_CFGR2_PREDIV_DIV2;
  RCC->CFGR = RCC_CFGR_PLLMUL8 | // 48Mhz (12MHz crystal)
              RCC_CFGR_PLLSRC_HSE_PREDIV;
  RCC->CR |= RCC_CR_PLLON;
  FLASH->ACR = _VAL2FLD(FLASH_ACR_LATENCY, 1) | FLASH_ACR_PRFTBE;
  //FLASH->ACR |= FLASH_ACR_LATENCY|FLASH_ACR_PRFTBE; // Активирование предвыборки
  while(! (RCC->CR & RCC_CR_PLLRDY));
  RCC->CFGR |= RCC_CFGR_SW_PLL;
}

void zi2c_init (void) {
  RCC->AHBENR |= RCC_AHBENR_GPIOBEN;
      
  RCC->APB1ENR |= RCC_APB1ENR_I2C1EN;
  RCC->CFGR3 |= RCC_CFGR3_I2C1SW;
      
  GPIOB->AFR[0] |= (1<<(4*6)) |(1<<(4*7));
  GPIOB->MODER &= ~(GPIO_MODER_MODER6 |GPIO_MODER_MODER7);
  GPIOB->MODER |= (GPIO_MODER_MODER6_1 |GPIO_MODER_MODER7_1);
  GPIOB->OTYPER |=(GPIO_OTYPER_OT_6 |GPIO_OTYPER_OT_7);
  I2C1->TIMINGR = (uint32_t)0x00B01A4B;
  I2C1->CR1 = I2C_CR1_PE;
}

uint8_t zi2c_write(uint8_t addr, uint8_t *data, uint8_t nbytes) {
  cntr = ttms;
  while(I2C1->ISR & I2C_ISR_BUSY) { if(ttms - cntr > I2CWAIT) return 0; }

  cntr = ttms;
  I2C1->CR2 = (nbytes << 16) | addr | I2C_CR2_AUTOEND | I2C_CR2_START; // autoend // now start transfer
  while(I2C1->CR2 & I2C_CR2_START) { if(ttms - cntr > I2CWAIT) return 0; }
    
  for(int i = 0; i < nbytes; ++i){
    cntr = ttms;
    while(!(I2C1->ISR & I2C_ISR_TXIS)){ if(ttms - cntr > I2CWAIT) return 0; }
    I2C1->TXDR = data[i]; // send data
  }
  while(I2C1->ISR & I2C_ISR_BUSY){ if(ttms - cntr > I2CWAIT) break; } // wait for data gone
  return 1;
}

uint8_t zi2c_read(uint8_t addr, uint8_t *data, uint8_t nbytes) {
  cntr = ttms;
  while (I2C1->ISR & I2C_ISR_BUSY) { if(ttms - cntr > I2CWAIT) return 0; }
  cntr = ttms;
  while (I2C1->CR2 & I2C_CR2_START) { if(ttms - cntr > I2CWAIT) return 0; }
  // read N bytes
  I2C1->CR2 = (nbytes<<16) | addr | 1 | I2C_CR2_AUTOEND | I2C_CR2_RD_WRN;
  I2C1->CR2 |= I2C_CR2_START;

  for (uint8_t i = 0; i < nbytes; ++i) {
    cntr = ttms;
    while (!(I2C1->ISR & I2C_ISR_RXNE)) { if(ttms - cntr > I2CWAIT) return 0; }
    *data++ = I2C1->RXDR;
  }
  return 1;
}

char *BcdByteToStr(uint8_t byte, char *buffer)
{
	//if (strnlen(&buffer[0]) >= 2) {
		buffer[0] = (byte >> 4) + 0x30;
		buffer[1] = (byte & 0b00001111) + 0x30;
		buffer[2] = '\0';
	//}
	return &buffer[0];
}

uint8_t StrToBcdByte(char *buffer)
{
	//if (strlen(&buffer[0]) < 2){
		//return 0;
	//}
	unsigned char bcdByteOld = buffer[0] - 0x30;
	bcdByteOld = bcdByteOld << 4;
	unsigned char bcdByteYoung = buffer[1] - 0x30;
	return bcdByteOld | bcdByteYoung;
}

///\\\

uint8_t dec2bcddiv(uint8_t data) 
{
  //Преобразование десятичного числа в BCD формат
  data = ((data/10)<<4)+(data%10);
  return data;
}

uint8_t dec2bcd2(uint8_t n)
{
  byte b = (n * 103) >> 10;
  return (b * 16 + n-(b*10));  
}

uint8_t dec2bcd4(uint8_t n)
{
  uint16_t a = n;
  byte b = (a*26) >> 8;  // b = (a*13) >> 7  was OK but slower!
  return  n + b*6;
}

uint8_t bcd2dec(uint8_t data)
{
  //Преобразование числа в BCD формате в десятичное число
    data = (data>>4)*10+(data & 0xF);
    return data;
}
// https://forum.arduino.cc/t/faster-dec2bcd-routine-especial-for-rtc-libraries/180741/7
uint8_t bcd2dec4(uint8_t n) { return n - 6 * (n >> 4); }

// Here is a fast version of bcd2bin().
uint8_t bcd2bin(uint8_t n) {
  uint8_t h = n >> 4;
  // n - 6 * (n >> 4)
  return n - (((h << 1) + h) << 1);
}

//_t.second = (((_t.second & 0b11110000) >> 4) * 10 + (_t.second & 0b00001111)); //Convert BCD to decimal
//_t.minute = (((_t.minute & 0b11110000) >> 4) * 10 + (_t.minute & 0b00001111));
//_t.hour   = (((_t.hour & 0b00110000) >> 4) * 10 + (_t.hour & 0b00001111));   //Convert BCD to decimal (assume 24 hour mode)
//day     = (day & B00000111); // 1-7 (ВС ПН ВТ СР ЧТ ПТ СУБ)
//date    = (((date & B00110000) >> 4) * 10 + (date & B00001111));     //Convert BCD to decimal  1-31
//month   = (((month & B00010000) >> 4) * 10 + (month & B00001111));   //msb7 is century overflow
//year    = (((year & B11110000) >> 4) * 10 + (year & B00001111));
