#include "rda5807.h"
#include "common.h"
#include "ee24c16.h"
#include "tm1637.h"
extern volatile uint8_t ispowerup;
extern volatile uint8_t  stNum;
extern volatile uint8_t  dispmode; // 0 - freq, 1 - volume
extern volatile uint16_t dispcntr; // задержка отображений не основного режима
extern volatile uint32_t ttms;
static volatile uint32_t cntr;
//#define  _RDA580X

#ifdef _RDS
#include "rds.h"
#endif

Tuner_type tuner;
static uint8_t rdBuf[14];
uint8_t *bufFM;

static uint8_t rda580xWriteI2C(uint8_t nbytes) 
{
  cntr = ttms;
  I2C1->CR2 = (nbytes << 16) | RDA5807M_I2C_ADDR | I2C_CR2_AUTOEND | I2C_CR2_START;
  while (I2C1->CR2 & I2C_CR2_START) { if (ttms - cntr > I2CWAIT) return 0; }

  for (int i = 0; i < nbytes; i++) {
    cntr = ttms;
    while (!(I2C1->ISR & I2C_ISR_TXIS)) { if (ttms - cntr > I2CWAIT) return 0; };
    I2C1->TXDR = wrBuf[i];
  }
  
  while (I2C1->ISR & I2C_ISR_BUSY) { if (ttms - cntr > I2CWAIT) break; } // wait for data gone
  return 1;
}
  

void rda580xInit(void)
{
  // if (tuner.ic == TUNER_RDA5807_DF)
  wrBuf[11] |= RDA5807_FREQ_MODE;
  return;
}

void rda580xSetFreq (void) {
uint16_t chan;
uint16_t fMin = RDA5807_MIN_FREQ;
uint8_t band = RDA580X_BAND_EASTEUROPE;
if (tuner.mono) wrBuf[0] |= RDA580X_MONO; else wrBuf[0] &= ~RDA580X_MONO;
#ifdef _RDS
    if (tuner.rds)
        wrBuf[1] |= RDA5807_RDS_EN;
    else
        wrBuf[0] &= ~RDA5807_RDS_EN;
#endif
  if (tuner.freq >= RDA5807_BAND_CHANGE_FREQ) {
      fMin = RDA5807_BAND_CHANGE_FREQ;
      band = RDA580X_BAND_US_EUROPE;
  }
  // Freq in grid
  chan = (tuner.freq - fMin) / RDA5807_CHAN_SPACING;
  wrBuf[2] = chan >> 2;                               // 8 MSB
  wrBuf[3] = ((chan & 0x03) << 6) | RDA580X_TUNE | band | RDA580X_SPACE_50;

  // Direct freq
  wrBuf[12] = ((tuner.freq - fMin) * 10) >> 8;
  wrBuf[13] = ((tuner.freq - fMin) * 10) & 0xFF;

  rda580xWriteI2C(RDA5807_WR_BYTES);

  return;
}

uint8_t *rda580xReadStatus (void) {
  uint8_t i;
  zi2c_read(RDA5807M_I2C_ADDR, rdBuf, sizeof(rdBuf));  //rdBuf[12];
  //u8 I2C_Read_Transaction (u8 Adress, u8 Register, u8 *Data, u8 Size);
  // Get RDS data
#ifdef _RDS
    if (tuner.rds) {
        // If RDS ready and sync flag are set 
        if ((rdBuf[0] & RDA5807_RDSR) && (rdBuf[0] & RDA5807_RDSS)) {
            // If there are no errors in blocks A and B
            if (!(rdBuf[3] & (RDA5807_BLERA | RDA5807_BLERB))) {
                // Send rdBuf[4..11] as 16-bit blocks A-D 
                rdsSetBlocks(&rdBuf[4]);
            }
        }
    }
#endif
  return rdBuf;
}

void rda580xSetAudio (void) {
  if (tuner.volume > RDA5807_VOL_MAX)
      tuner.volume = RDA5807_VOL_MAX;
  if (tuner.volume < RDA5807_VOL_MIN)
      tuner.volume = RDA5807_VOL_MIN;
  if (tuner.mute || !tuner.volume)
      wrBuf[0] &= ~RDA580X_DMUTE;
  else
      wrBuf[0] |= RDA580X_DMUTE;
  if (tuner.bass)
      wrBuf[0] |= RDA5807_BASS;
  else
      wrBuf[0] &= ~RDA5807_BASS;
  wrBuf[3] &= ~RDA580X_TUNE;
  if (tuner.volume)
      wrBuf[7] = RDA580X_LNA_PORT_SEL_LNAP | (tuner.volume - 1);
  rda580xWriteI2C(RDA5807_WR_BYTES);
}

void rda580xPowerOn (void) {
  wrBuf[1] |= RDA580X_ENABLE;
  rda580xWriteI2C(RDA5807_WR_BYTES);
  return;
}

void rda580xPowerOff (void) {
  wrBuf[1] &= ~RDA580X_ENABLE;
  rda580xWriteI2C(RDA5807_WR_BYTES);
  return;
}


//
//# TUNER
//

void tunerSetFreq () {
  if (tuner.freq < tuner.fMin)
      tuner.freq = tuner.fMin;
  else if (tuner.freq > tuner.fMax)
      tuner.freq = tuner.fMax;
  rda580xSetFreq();
#ifdef _RDS
    rdsDisable(); // Clear RDS buffer
#endif
  return;
}

void tunerChangeFreq (int8_t mult) {
  if ((tuner.freq > FM_BAND_DIV_FREQ) || (mult > 0 && tuner.freq == FM_BAND_DIV_FREQ))
       tuner.freq += tuner.step2 * mult;
  else
       tuner.freq += tuner.step1 * mult;
  tunerSetFreq();
  tunerSetFreq();
  return;
}

void tunerReadStatus(void) {
  bufFM = rda580xReadStatus();
  return;
}

void tunerSetMono (uint8_t value) {
  tuner.mono = value;
  tunerSetFreq();
  return;
}

uint8_t tunerStereo (void) {
  uint8_t ret = !tuner.mono;
  ret = RDA5807_BUF_STEREO(bufFM);
  return ret;
}

uint8_t tunerLevel (void) {
  uint8_t ret = 0;
  ret = (bufFM[2] & RDA580X_RSSI) >> 1;
  if (ret < 24)
    ret = 0;
  else
    ret = (ret - 24) >> 1;
  if (tunerStereo())
    ret = 13;
  else
    ret = 3;
  return ret;
}

void tunerSetVolume (int8_t value) {
  tuner.volume = value;
  rda580xSetAudio();
  return;
}

void tunerSetMute (uint8_t value) {
 tuner.mute = value;
 rda580xSetAudio();
 return;
}

void tunerSetBass (uint8_t value) {
  tuner.bass = value;
  rda580xSetAudio();
}

void tunerPowerOn(void)
{
  eeprom_load_tuner_data();
  rda580xPowerOn();
  tunerSetFreq();
  tunerSetMute(0);
  rda580xSetAudio();
  tunerSetFreq();
  ispowerup = 1;
  tm_set_brightness(2);
  _delay_ms(2000);
  eeprom_load_tuner_station(stNum);
  dispmode = 1; dispcntr = 40;
}

void tunerPowerOff(void)
{
  tm_set_brightness(0);
  eeprom_save_tuner_data();
  tunerSetVolume(0);
  tunerSetMute(1);
  rda580xPowerOff();
  ispowerup = 0;
  _delay_ms(10000);
}
//###########################################################################//
void eeprom_load_tuner_data()
{
  uint8_t tmpBuf [16];
  ee24c16rp(0, tmpBuf);
  tuner.ctrl   =  tmpBuf[ 0];
  tuner.step1  =  tmpBuf[ 1];
  tuner.step2  =  tmpBuf[ 2];
  tuner.fMin   = (tmpBuf[ 3] << 8) | tmpBuf[ 4]; //(a << 8) | b
  tuner.fMax   = (tmpBuf[ 5] << 8) | tmpBuf[ 6];
  tuner.freq   = (tmpBuf[ 7] << 8) | tmpBuf[ 8];
  tuner.mono   =  tmpBuf[ 9];
  tuner.rds    =  tmpBuf[10];
  tuner.volume =  tmpBuf[11];
  tuner.bass   =  tmpBuf[12];
  tuner.rdFreq = (tmpBuf[13] << 8) | tmpBuf[14];
  tuner.mute   =  tmpBuf[15];
}

// сохраняем на 1-ую страницу
void eeprom_save_tuner_data(void) 
{
  uint8_t tmpBuf [16];
  tmpBuf[ 0] = tuner.ctrl;
  tmpBuf[ 1] = tuner.step1;
  tmpBuf[ 2] = tuner.step2;
  tmpBuf[ 4] = tuner.fMin; tmpBuf[ 3] = tuner.fMin >> 8;  
  tmpBuf[ 6] = tuner.fMax; tmpBuf[ 5] = tuner.fMax >> 8;
  tmpBuf[ 8] = tuner.freq; tmpBuf[ 7] = tuner.freq >> 8;  
  tmpBuf[ 9] = tuner.mono;
  tmpBuf[10] = tuner.rds;
  tmpBuf[11] = tuner.volume;
  tmpBuf[12] = tuner.bass;
  tmpBuf[14] = tuner.rdFreq; tmpBuf[13] = tuner.rdFreq >> 8;
  tmpBuf[15] = tuner.mute;  
  ee24c16wp(0, tmpBuf);
}

// stations 0..7  // страниц для сохранения 2, 3, 4
void eeprom_save_tuner_station (uint8_t station) 
{
  uint8_t tmpBuf [16];
  uint8_t eepage = 3; //
  uint8_t wnum [8] = {0, 2, 4, 6, 8, 10, 12, 14};
  if (station > 7) { eepage = 4; station = station - 8; }

  ee24c16rp(eepage, tmpBuf);
  tmpBuf[wnum[station]    ] = highByte(tuner.freq);
  tmpBuf[wnum[station] + 1] = lowByte (tuner.freq);

  ee24c16wp(eepage, tmpBuf);
}

// stations 0..7 ... ??
void eeprom_load_tuner_station (uint8_t station) 
{
  uint8_t tmpBuf [16];
  uint8_t eepage = 3; // 01234567 8
  uint8_t wnum [8] = {0, 2, 4, 6, 8, 10, 12, 14};
  if (station > 7) { eepage = 4; station = station - 8; } 

  ee24c16rp(eepage, tmpBuf);
  tuner.freq   = (tmpBuf[wnum[station]] << 8) | tmpBuf[wnum[station]+1]; //(a << 8) | b
  tunerSetFreq();
  tunerSetFreq();
}

// тут пока под вопросом
// RDA5807M_I2C_ADDR
uint8_t rda5807_write16_reg (uint8_t reg, uint16_t data)
{
  cntr = ttms;
  I2C1->CR2 =  I2C_CR2_AUTOEND | (3<<16) | (0x11<<1); 
  while (!(I2C1->ISR & I2C_ISR_TXE)) { if(ttms - cntr > I2CWAIT) return 0; } // Check Tx empty

  I2C1->TXDR = reg; // Byte to send
  I2C1->CR2 |= I2C_CR2_START; // Go
  while (!(I2C1->ISR & I2C_ISR_TXIS)) { if(ttms - cntr > I2CWAIT) return 0; }

  I2C1->TXDR = (uint8_t)(data >> 8); // Byte to send
  while (!(I2C1->ISR & I2C_ISR_TXIS)) { if(ttms - cntr > I2CWAIT) return 0; }

  I2C1->TXDR = (uint8_t)(data & 0x00FF); // Byte to send
  while (!(I2C1->ISR & I2C_ISR_TXIS)) { if(ttms - cntr > I2CWAIT) return 0; }
  return 1;
}

uint8_t rda5807_write16_reg2 (uint8_t reg, uint8_t *buff, uint8_t nbytes)
{
  cntr = ttms;
  I2C1->CR2 = (nbytes << 16) | RDA5807M_I2C_ADDR | I2C_CR2_AUTOEND | I2C_CR2_START;
  for (int i = 0; i < nbytes; ++i){
    cntr = ttms;
    while(!(I2C1->ISR & I2C_ISR_TXIS)) { if(ttms - cntr > I2CWAIT) return 0; };
    I2C1->TXDR = buff[i];
  }
  while(I2C1->ISR & I2C_ISR_BUSY) { if(ttms - cntr > I2CWAIT) break; } // wait for data gone
  return 1;
}

uint16_t rda5807_read16_reg (uint8_t reg) 
{
  uint16_t temp = 0;

  cntr = ttms;
  I2C1->CR2 =  (1<<16) | (0x10<<1); 
  while (!(I2C1->ISR & I2C_ISR_TXE)) { if(ttms - cntr > I2CWAIT) return 0; }

  I2C1->TXDR = reg; // Byte to send
  I2C1->CR2 |= I2C_CR2_START; // Go
  while (!(I2C1->ISR & I2C_ISR_TC)) { if(ttms - cntr > I2CWAIT) return 0; }
  
  I2C1->CR2 =  I2C_CR2_AUTOEND | (2<<16) | (RDA5807M_I2C_ADDR<<1) | I2C_CR2_RD_WRN | I2C_CR2_NACK; 
  I2C1->CR2 |= I2C_CR2_START; // Go
  while (!(I2C1->ISR & I2C_ISR_RXNE)) { if(ttms - cntr > I2CWAIT) return 0; }
  
  temp  = (uint16_t)(I2C1->RXDR <<8);
  while (!(I2C1->ISR & I2C_ISR_RXNE)) { if(ttms - cntr > I2CWAIT) return 0; }
  
  temp |=  (uint16_t)I2C1->RXDR;
  while((I2C1->ISR & I2C_ISR_RXNE)==0) { if(ttms - cntr > I2CWAIT) return 0; }
  return temp;
}