#include "common.h"
#include "usart.h"

// 
void usart_putc(uint8_t data)
{
  while(!(USART1->ISR & USART_ISR_TC));
  USART1->TDR = data;
}

void usart_puts(const char * data)
{
  while((*data )) { 
    usart_putc(*data);
    data++;
  }
}

// Transmitting Decimal Values
void usart_put_u08(uint8_t val)
{
  uint8_t dig1 = '0', dig2 = '0';
  // count value in 100s place
  while(val >= 100) {
      val -= 100;
      dig1++;
  }
  // count value in 10s place
  while(val >= 10) {
      val -= 10;
      dig2++;
  }
  // print first digit (or ignore leading zeros)
  if(dig1 != '0') usart_putc(dig1);
  // print second digit (or ignore leading zeros)
  if((dig1 != '0') || (dig2 != '0')) usart_putc(dig2);
  // print final digit
  usart_putc(val + '0');
}

void usart_put_u16(uint16_t val)
{
  uint8_t dig1 = '0', dig2 = '0', dig3 = '0', dig4 = '0';
  // count value in 10000s place
  while(val >= 10000) {
      val -= 10000;
      dig1++;
  }
  // count value in 1000s place
  while(val >= 1000) {
      val -= 1000;
      dig2++;
  }
  // count value in 100s place
  while(val >= 100) {
      val -= 100;
      dig3++;
  }
  // count value in 10s place
  while(val >= 10) {
      val -= 10;
      dig4++;
  }
  // was previous value printed?
  uint8_t prevPrinted = 0;
  // print first digit (or ignore leading zeros)
  if(dig1 != '0') {usart_putc(dig1); prevPrinted = 1;}
  // print second digit (or ignore leading zeros)
  if(prevPrinted || (dig2 != '0')) {usart_putc(dig2); prevPrinted = 1;}
  // print third digit (or ignore leading zeros)
  if(prevPrinted || (dig3 != '0')) {usart_putc(dig3); prevPrinted = 1;}
  // print third digit (or ignore leading zeros)
  if(prevPrinted || (dig4 != '0')) {usart_putc(dig4); prevPrinted = 1;}
  // print final digit
  usart_putc(val + '0');
}

/*
 29    можно так же до разрешения UART сбросить флаги прерываний
 30 1. включить тактирование порта
 31 2. установить альтернативные функции порта
 32 3. установить режим USART
 33 4. установить разрешение прерывания в USART
 34 5. разрешить прерывание USART в NVIC
 35 6. разрешить работу USART
 36 7. установить приоритет прерывания в NVIC
 37 */
void usart_init(void)
{
  ////
  //\\RCC->AHBENR    |= RCC_AHBENR_GPIOAEN;         //включаем тактирование GPIOA
  RCC->APB2ENR   |= RCC_APB2ENR_USART1EN;       //включаем тактирование USART1
  // по умолчанию USART для STM32F030 A3 и A3
  GPIOA->MODER   |= GPIO_MODER_MODER9_1;        //PA9  (TX) функция для пина A9 не основная, поэтому настраиваем как           альтернативную
  GPIOA->MODER   |= GPIO_MODER_MODER10_1;       //PA10 (RX) функция для пина A10 не основная, поэтому настраиваем как          альтернативную
  GPIOA->OSPEEDR |= GPIO_OSPEEDER_OSPEEDR9_0;   //PA9  (TX) - 10 MHz
  // GPIOA->AFR[1]    |= 0x0110;  //
  // По первой таблице мы находим что UART1 для пинов PA9 и PA10 соответствует AF1.
  // По второму скрину видим что AF1 соответствует 0001 бинарное или 0х01.
  // Вот это самой число и надо занести в тетрады AFRH10 и AFRH9, что и делает сей код:
  GPIOA->AFR[1] = (1<<(2*4)) | (1<<(1*4));   // или что то же самое GPIOA->AFR[1]|= 0x0110; записываем 0b0001·
  // в AFRH10 и AFRH9 (UART1 для пинов PA9 и PA10 соответствует AF1, AF1 соответствует 0b0001)
  RCC->CFGR3  &= ~RCC_CFGR3_USART1SW;
  //RCC->CFGR3  |=  RCC_CFGR3_USART1SW_0;         //System clock (SYSCLK) selected as USART1 clock
  //RCC->CFGR3  |=  RCC_CFGR3_USART1SW;         //HSI selected as USART1 clock
  //RCC->CFGR3  |=  RCC_CFGR3_USART1SW_LSE;
  RCC->CFGR3  |=  RCC_CFGR3_USART1SW_PCLK;      //

  USART1->CR1 |= USART_CR1_UE;                  //включаем USART1
  USART1->CR1 &= ~USART_CR1_M;                  //данные - 8 бит
  USART1->CR2 &= ~USART_CR2_STOP;               //1 стоп-бит
  //USART1->BRR = (APBCLK+BAUDRATE/2) / BAUDRATE; //скорость usart
  //USART1->BRR =(APBCLK+BAUDRATE/2)/BAUDRATE;
  //USART1->BRR = (APBCLK/BAUDRATE)/16;
  USART1->BRR = FCPU/(BAUDRATE/2);
  //С BRR всё предельно просто:
  //BRR = F_USART / baud;
  //И всё.  BRR = 36 000 000 / 19200 = 1875 = 0x753;
  //16 тиков на бод и 16 градаций в дробной части не просто так сделаны.

  USART1->CR1 |= USART_CR1_TE;                  //включаем передатчик USART1
  USART1->CR1 |= USART_CR1_RE;                  //включаем приемник USART1
  USART1->CR1 |= USART_CR1_RXNEIE;              //прерывание по приему данных
  NVIC_EnableIRQ (USART1_IRQn);                 //разрешить прерывания от USART1
  //  Вначале в USART разрешаются прерывания по определённому событию:
  //USART1->CR1  |= USART_CR1_RXNEIE;       //прерывание по приему данных
  // Затем в NVIC разрешается прерывание USART
  //NVIC_EnableIRQ (USART1_IRQn);           //разрешить прерывания от USART1
  // Затем в прерывании анализируется источник прерывания:
  //if((USART1->ISR & USART_ISR_RXNE)!=0)
  // и принимается решение что-то делать.                                                                                  
  //------------------------------
  //GPIOA->AFR[0] |= _VAL2FLD(GPIO_AFRL_AFSEL2, 1) | _VAL2FLD(GPIO_AFRL_AFSEL3, 1); // GPIO_AFRL_AFSEL2 - это номер ножки,     число 1 это (0001: AF1)
  // Для UART A2 A3 на STM32F030
}

void usart_put_hexu(uint32_t Value, uint8_t NumBytes)
{
  uint8_t i;
  uint8_t t;
  //Every byte is two nibbles.
  NumBytes *= 2;
  //Loop for every nibble in a 32 bit integer.
  for (i = 0; i < 8; i++) {
    //Only print the nibbles we have to.
    if ((8 - i) <= NumBytes) {
      //Extract the nibble for printing out of Value.
      t = (Value & 0xF0000000) >> 28;
      //Convert to ascii and print
      usart_putc(t < 10 ? t + 48 : t < 16 ? t + 55 : '?');
    }
    //Shift the next nibble into position.
    Value <<= 4;
  }
  //Append 'h' to make clear it is a HEX value.
  usart_putc('h');
}

void usart_put_hex(int32_t Value, uint8_t NumBytes)
{
  //See if the number is negative, print a minus sign if it is.
  if (Value < 0) usart_putc('-');
  //Remove the sign and print.
  usart_put_hexu(Value < 0 ? 0 - Value : Value, NumBytes);
}